==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/client/render/block_entity/PetalApothecaryBlockEntityRenderer.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.render.block_entity;

import net.minecraft.class_1058;
import net.minecraft.class_1159;
import net.minecraft.class_1160;
import net.minecraft.class_1163;
import net.minecraft.class_1799;
import net.minecraft.class_2246;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_4722;
import net.minecraft.class_5614;
import net.minecraft.class_776;
import net.minecraft.class_809;
import net.minecraft.class_827;
import org.jetbrains.annotations.NotNull;

import vazkii.botania.api.block.PetalApothecary;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.common.block.block_entity.PetalApothecaryBlockEntity;

public class PetalApothecaryBlockEntityRenderer implements class_827<PetalApothecaryBlockEntity> {
	private final class_776 blockRenderDispatcher;

	public PetalApothecaryBlockEntityRenderer(class_5614.class_5615 context) {
		this.blockRenderDispatcher = context.method_32141();
	}

	@Override
	public void render(@NotNull PetalApothecaryBlockEntity altar, float pticks, class_4587 ms, class_4597 buffers, int light, int overlay) {
		ms.method_22903();
		ms.method_22904(0.5, 1.25, 0.5);

		boolean water = altar.getFluid() == PetalApothecary.State.WATER;
		boolean lava = altar.getFluid() == PetalApothecary.State.LAVA;
		if (water || lava) {
			ms.method_22903();
			float v = 1F / 8F;

			if (water) {
				int petals = 0;
				for (int i = 0; i < altar.inventorySize(); i++) {
					if (!altar.getItemHandler().method_5438(i).method_7960()) {
						petals++;
					} else {
						break;
					}
				}

				if (petals > 0) {
					final float modifier = 6F;
					final float rotationModifier = 0.25F;
					final float radiusBase = 1.2F;
					final float radiusMod = 0.1F;

					double ticks = (ClientTickHandler.ticksInGame + pticks) * 0.5;
					float offsetPerPetal = 360F / petals;

					ms.method_22903();
					ms.method_22904(-0.05F, -0.38F, 0F);
					ms.method_22905(v, v, v);
					for (int i = 0; i < petals; i++) {
						float offset = offsetPerPetal * i;
						float deg = (int) (ticks / rotationModifier % 360F + offset);
						float rad = deg * (float) Math.PI / 180F;
						float radiusX = (float) (radiusBase + radiusMod * Math.sin(ticks / modifier));
						float radiusZ = (float) (radiusBase + radiusMod * Math.cos(ticks / modifier));
						float x = (float) (radiusX * Math.cos(rad));
						float z = (float) (radiusZ * Math.sin(rad));
						float y = (float) Math.cos((ticks + 50 * i) / 5F) / 10F;

						ms.method_22903();
						ms.method_22904(x, y, z);
						float xRotate = (float) Math.sin(ticks * rotationModifier) / 2F;
						float yRotate = (float) Math.max(0.6F, Math.sin(ticks * 0.1F) / 2F + 0.5F);
						float zRotate = (float) Math.cos(ticks * rotationModifier) / 2F;

						v /= 2F;
						ms.method_22904(v, v, v);
						ms.method_22907(new class_1160(xRotate, yRotate, zRotate).method_23214(deg));
						ms.method_22904(-v, -v, -v);
						v *= 2F;

						class_1799 stack = altar.getItemHandler().method_5438(i);
						class_310.method_1551().method_1480().method_23178(stack, class_809.class_811.field_4318,
								light, overlay, ms, buffers, 0);
						ms.method_22909();
					}

					ms.method_22909();
				}
			}

			float alpha = lava ? 1F : 0.7F;

			ms.method_22904(-8 / 16F, -0.3125F, -8 / 16F);
			ms.method_22907(class_1160.field_20703.method_23214(90));
			ms.method_22905(1 / 16F, 1 / 16F, 1 / 16F);

			class_1058 sprite = lava ? this.blockRenderDispatcher.method_3349(class_2246.field_10164.method_9564()).method_4711()
					: this.blockRenderDispatcher.method_3349(class_2246.field_10382.method_9564()).method_4711();
			int color = lava ? -1
					: class_1163.method_4961(altar.method_10997(), altar.method_11016());
			class_4588 buffer = buffers.getBuffer(class_4722.method_24076());
			renderIcon(ms, buffer, sprite, color, alpha, overlay, lava ? 0xF000F0 : light);
			ms.method_22909();
		}
		ms.method_22909();
	}

	private void renderIcon(class_4587 ms, class_4588 builder, class_1058 sprite, int color, float alpha, int overlay, int light) {
		int red = ((color >> 16) & 0xFF);
		int green = ((color >> 8) & 0xFF);
		int blue = (color & 0xFF);
		class_1159 mat = ms.method_23760().method_23761();
		int start = 3;
		int end = 13;
		builder.method_22918(mat, start, end, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(start), sprite.method_4570(end)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, end, end, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(end), sprite.method_4570(end)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, end, start, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(end), sprite.method_4570(start)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, start, start, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(start), sprite.method_4570(start)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
	}

}

==> Botania-1.20.1-451-FABRIC-20250920.075237-9-sources.jar: vazkii/botania/client/render/block_entity/PetalApothecaryBlockEntityRenderer.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.client.render.block_entity;

import net.minecraft.class_1058;
import net.minecraft.class_1163;
import net.minecraft.class_1799;
import net.minecraft.class_2246;
import net.minecraft.class_310;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_4722;
import net.minecraft.class_5614;
import net.minecraft.class_776;
import net.minecraft.class_811;
import net.minecraft.class_827;
import org.jetbrains.annotations.NotNull;
import org.joml.Matrix4f;
import org.joml.Quaternionf;

import vazkii.botania.api.block.PetalApothecary;
import vazkii.botania.client.core.handler.ClientTickHandler;
import vazkii.botania.common.block.block_entity.PetalApothecaryBlockEntity;
import vazkii.botania.common.helper.VecHelper;

public class PetalApothecaryBlockEntityRenderer implements class_827<PetalApothecaryBlockEntity> {
	private final class_776 blockRenderDispatcher;

	public PetalApothecaryBlockEntityRenderer(class_5614.class_5615 context) {
		this.blockRenderDispatcher = context.method_32141();
	}

	@Override
	public void render(@NotNull PetalApothecaryBlockEntity altar, float pticks, class_4587 ms, class_4597 buffers, int light, int overlay) {
		ms.method_22903();
		ms.method_22904(0.5, 1.25, 0.5);

		boolean water = altar.getFluid() == PetalApothecary.State.WATER;
		boolean lava = altar.getFluid() == PetalApothecary.State.LAVA;
		if (water || lava) {
			ms.method_22903();
			float v = 1F / 8F;

			if (water) {
				int petals = 0;
				for (int i = 0; i < altar.inventorySize(); i++) {
					if (!altar.getItemHandler().method_5438(i).method_7960()) {
						petals++;
					} else {
						break;
					}
				}

				if (petals > 0) {
					final float modifier = 6F;
					final float rotationModifier = 0.25F;
					final float radiusBase = 1.2F;
					final float radiusMod = 0.1F;

					double ticks = (ClientTickHandler.ticksInGame + pticks) * 0.5;
					float offsetPerPetal = 360F / petals;

					ms.method_22903();
					ms.method_46416(-0.05F, -0.38F, 0F);
					ms.method_22905(v, v, v);
					for (int i = 0; i < petals; i++) {
						float offset = offsetPerPetal * i;
						float deg = (int) (ticks / rotationModifier % 360F + offset);
						float rad = VecHelper.toRadians(deg);
						float radiusX = (float) (radiusBase + radiusMod * Math.sin(ticks / modifier));
						float radiusZ = (float) (radiusBase + radiusMod * Math.cos(ticks / modifier));
						float x = (float) (radiusX * Math.cos(rad));
						float z = (float) (radiusZ * Math.sin(rad));
						float y = (float) Math.cos((ticks + 50 * i) / 5F) / 10F;

						ms.method_22903();
						ms.method_46416(x, y, z);
						float xRotate = (float) Math.sin(ticks * rotationModifier) / 2F;
						float yRotate = (float) Math.max(0.6F, Math.sin(ticks * 0.1F) / 2F + 0.5F);
						float zRotate = (float) Math.cos(ticks * rotationModifier) / 2F;

						v /= 2F;
						ms.method_46416(v, v, v);
						ms.method_22907(new Quaternionf().rotateAxis(rad, xRotate, yRotate, zRotate));
						ms.method_46416(-v, -v, -v);
						v *= 2F;

						class_1799 stack = altar.getItemHandler().method_5438(i);
						class_310.method_1551().method_1480().method_23178(stack, class_811.field_4318,
								light, overlay, ms, buffers, altar.method_10997(), 0);
						ms.method_22909();
					}

					ms.method_22909();
				}
			}

			float alpha = lava ? 1F : 0.7F;

			ms.method_46416(-8 / 16F, -0.3125F, -8 / 16F);
			ms.method_22907(VecHelper.rotateX(90));
			ms.method_22905(1 / 16F, 1 / 16F, 1 / 16F);

			class_1058 sprite = lava ? this.blockRenderDispatcher.method_3349(class_2246.field_10164.method_9564()).method_4711()
					: this.blockRenderDispatcher.method_3349(class_2246.field_10382.method_9564()).method_4711();
			int color = lava ? -1
					: class_1163.method_4961(altar.method_10997(), altar.method_11016());
			class_4588 buffer = buffers.getBuffer(class_4722.method_24076());
			renderIcon(ms, buffer, sprite, color, alpha, overlay, lava ? 0xF000F0 : light);
			ms.method_22909();
		}
		ms.method_22909();
	}

	private void renderIcon(class_4587 ms, class_4588 builder, class_1058 sprite, int color, float alpha, int overlay, int light) {
		int red = ((color >> 16) & 0xFF);
		int green = ((color >> 8) & 0xFF);
		int blue = (color & 0xFF);
		Matrix4f mat = ms.method_23760().method_23761();
		int start = 3;
		int end = 13;
		builder.method_22918(mat, start, end, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(start), sprite.method_4570(end)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, end, end, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(end), sprite.method_4570(end)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, end, start, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(end), sprite.method_4570(start)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
		builder.method_22918(mat, start, start, 0).method_1336(red, green, blue, (int) (alpha * 255F))
				.method_22913(sprite.method_4580(start), sprite.method_4570(start)).method_22922(overlay).method_22916(light).method_22914(0, 0, 1).method_1344();
	}

}
